-- cKick Bar Module by Chimaine
-- Simple bar module, ripped and modified from LibCandyBar-3.0

local addon, db = "cKick", "cKickDB"

local function debug(...)
	if ( cKick.Debug ) then
		print(...)
	end
end

local function print(...)
	local output = cKick.Name .. ": "
	
	for i = 1, select("#", ...) do
		output = output .. tostring(select(i, ...)) .. " "
	end

	DEFAULT_CHAT_FRAME:AddMessage(output .. ".")
end

local LSM = LibStub("LibSharedMedia-3.0")

local id = 1

-- Time formatting
local tformat1 = "%d:%02d"
local tformat2 = "%1.1f"
local tformat3 = "%.0f"

local function SecondsToTimeDetail( t )
	if t >= 3600 then -- > 1 hour
		local h = floor(t/3600)
		local m = t - (h*3600)
		return tformat1, h, m
	elseif t >= 60 then -- 1 minute to 1 hour
		local m = floor(t/60)
		local s = t - (m*60)
		return tformat1, m, s
	elseif t < 10 then -- 0 to 10 seconds
		return tformat2, t
	else -- 10 seconds to one minute
		return tformat3, floor(t + .5)
	end
end

function cKick:CreateBar(width, height)
	local bar = CreateFrame("Frame", nil, UIParent)
	
	bar:SetWidth(width)
	bar:SetHeight(height)
	
	bar.Id = id
	id = id + 1
	
	bar.duration = 0
	bar.remaining = 0
	bar.exp = 0
	bar.isRunning = false
	bar.hideOnStop = false
	
	bar:SetScale(1)
	bar:SetAlpha(1)

	local statusbar = CreateFrame("StatusBar", nil, bar)
	statusbar:SetAllPoints()
	statusbar:SetStatusBarTexture(LSM:Fetch("statusbar", _G[db]["barTexture"]))
	bar.Bar = statusbar
	
	local bg = statusbar:CreateTexture(nil, "BACKGROUND")
	bg:SetAllPoints()
	bg:SetTexture(LSM:Fetch("statusbar", _G[db]["barTexture"]))
	bg:SetVertexColor(0.5, 0.5, 0.5, 0.5)
	bar.Background = bg
	
	local timer = statusbar:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmallOutline")
	timer:SetPoint("RIGHT", statusbar, -2, 0)
	bar.Timer = timer
	
	local label = statusbar:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmallOutline")
	label:SetPoint("LEFT", statusbar, 2, 0)
	label:SetPoint("RIGHT", statusbar, -2, 0)
	label:SetTextColor(1,1,1,1)
	label:SetJustifyH("CENTER")
	label:SetJustifyV("MIDDLE")
	bar.Label = label
	
	local arrowLeft = statusbar:CreateTexture(nil, "OVERLAY")
	arrowLeft:SetPoint("RIGHT", statusbar, "LEFT", 0, 0)
	arrowLeft:SetTexture("Interface\\Addons\\cKick\\media\\arrowLeft.tga")
	arrowLeft:SetVertexColor(1, .1, .1)
	arrowLeft:SetWidth(height)
	arrowLeft:SetHeight(height)
	arrowLeft:Hide()
	bar.arrowLeft = arrowLeft
	
	local arrowRight = statusbar:CreateTexture(nil, "OVERLAY")
	arrowRight:SetPoint("LEFT", statusbar, "RIGHT", 0, 0)
	arrowRight:SetTexture("Interface\\Addons\\cKick\\media\\arrowRight.tga")
	arrowRight:SetVertexColor(1, .1, .1)
	arrowRight:SetWidth(height)
	arrowRight:SetHeight(height)
	arrowRight:Hide()
	bar.arrowRight = arrowRight
	
	bar.BarOnUpdate = function(self)
		local t = GetTime()
		if t >= self.exp then
			self.Bar:SetValue(0)
			self:Stop()
		else
			self.remaining = self.exp - t
			self.Bar:SetValue(self.remaining)
			self.Timer:SetFormattedText(SecondsToTimeDetail(self.remaining))
			if self.funcs then
				for i, v in ipairs(self.funcs) do v(self) end
			end
		end
	end
	
	bar.BarOnUpdateInvert = function(self)
		local t = GetTime()
		if t >= self.exp then
			self.Bar:SetValue(self.duration)
			self:Stop()
		else
			self.remaining = self.exp - t
			self.Bar:SetValue(self.duration - self.remaining)
			self.Timer:SetFormattedText(SecondsToTimeDetail(self.remaining))
			if self.funcs then
				for i, v in ipairs(self.funcs) do v(self) end
			end
		end
	end
	
	bar.ShowArrows = function(self)
		self.arrowLeft:Show()
		self.arrowRight:Show()
	end
	
	bar.HideArrows = function(self)
		self.arrowLeft:Hide()
		self.arrowRight:Hide()
	end
	
	bar.IsArrowShown = function(self)
		if ( self.arrowLeft:IsShown() and self.arrowRight:IsShown() ) then
			return true
		end
	end

	bar.SetLabel = function(self, text)
		self.Label:SetText(text)
	end
	
	bar.SetTextStyle = function(self, modFont, modSize, modFlags)
		local font, size, flags = self.Label:GetFont()
		self.Label:SetFont(modFont or font, modSize or size, modFlags or flags)
		
		font, size, flags = self.Timer:GetFont()
		self.Timer:SetFont(modFont or font, modSize or size, modFlags or flags)
	end
	
	bar.SetColor = function(self, r, g, b, a)
		self.Bar:SetStatusBarColor(r, g, b, a)
	end
	
	bar.Start = function(self, duration, invert, hideOnStop)
		debug("Start bar", duration, "| Invert:", invert)
		self.duration = duration
		self.remaining = duration
		self.exp = GetTime() + duration
		self.hideOnStop = hideOnStop
		
		self.Bar:SetMinMaxValues(0, duration)

		self:SetScript("OnUpdate", invert and self.BarOnUpdateInvert or self.BarOnUpdate)
		self.Timer:Show()
		
		self:Show()
		
		self.isRunning = true
	end
	
	bar.Stop = function(self, hide)
		if ( hide or self.hideOnStop) then
			self:Hide()
		end
		
		self.duration = 0
		self.remaining = 0
		self.exp = 0
		self.hideOnStop = false
		
		self:SetScript("OnUpdate", nil)
		self.Timer:Hide()
		
		self.isRunning = false
	end
	
	return bar
end

