--[[
	cKick
	Copyright(c) 2009, Tobias 'Chimaine' Rummelt, postmaster(at)tobis-little-page.de
	Chimaine on EU-Destromath
	All rights reserved.
]]
--[[
	Thanks to Sakuri, Raymakur, Chalya and Manidin of EU-Destromath for early alpha testing
	and my guild Karma for beta testing :)
]]

local addon, db = "cKick", "cKickDB"

local _G = getfenv(0)

_G[addon] = CreateFrame("Frame", addon)

_G[addon].Name = GetAddOnMetadata(addon, "Title")
_G[addon].Version = GetAddOnMetadata(addon, "Version")

_G[addon].DBVersion = 1

local debug = false
_G[addon].debug = function(...)
	if ( debug ) then
		_G[addon].print("DEBUG:", ...)
	end
end

_G[addon].print = function(...)
	local output = _G[addon].Name .. ":"

	for i = 1, select("#", ...) do
		output = output .. " " .. tostring(select(i, ...))
	end

	DEFAULT_CHAT_FRAME:AddMessage(output .. ".")
end

------------------------------------------------------
-- Begin addon specific stuff

local LSM = LibStub("LibSharedMedia-3.0")
LSM:Register("statusbar", "darkborder", "Interface\\Addons\\cKick\\media\\darkborder.tga") 

cKick.Bars = {}
cKick.Kicker = {}

local type = _G.type
local next = _G.next
local unpack = _G.unpack
local select = _G.select
local tonumber = _G.tonumber
local tostring = _G.tostring

local UnitName = _G.UnitName
local IsRaidLeader = _G.IsRaidLeader
local IsRaidOfficer = _G.IsRaidOfficer
local SendAddonMessage = _G.SendAddonMessage
local GetNumRaidMembers = _G.GetNumRaidMembers

local _syncCmd = {
	prefix = "CKICK_SYNC",
	kickerSync = "KICKER_SYNC",
	rotationSync = "ROTATION_SYNC",
	rotationReset = "ROTATION_RESET",
}

local _raidList = {}
local _kicks = {
	["DEATHKNIGHT"] = {
		[47528] = { 10, 4 }, -- Mind Freeze
		[32747] = { 120, 5, true }, -- Strangulate Interrupt Effect
	},
	["DRUID"] = {
		[49802] = { 10, 3 }, -- Maim
		[19675] = { 15, 4, true }, -- Feral Charge
	},
	["HUNTER"] = {
		[32747] = { 20, 3, true }, -- Silence Shot Interrupt Effect
	},
	["MAGE"] = {
		[2139] = { 24, 8 }, -- Counterspell
	},
	["PALADIN"] = {
		[32747] = { 60, 3, true }, -- Hammer of Justice Interrupt Effect
	},
	["PRIEST"] = {
		[1] = { 1, 1, true }, -- dummy
	},
	["ROGUE"] = {
		[1766] = { 10, 5 }, -- Kick
	},
	["SHAMAN"] = {
		[57994] = { 6, 2 }, -- Wind Shear
	},
	["WARRIOR"] = {
		[72] = { 12, 6 }, -- Shield Bash
		[6552] = { 10, 4 }, -- Pummel
	},
	["WARLOCK"] = {
		[1] = { 1, 1, true }, -- dummy
	},
}

function cKick:CanSendSync()
	if ( debug == true ) then
		self.debug("Debug modus enabled, fake Raidlead")
		return true
	end
	
	if ( self:IsInRaid() ) then
		if ( IsRaidLeader() or IsRaidOfficer() ) then
			self.debug("You can send syncs")
			return true
		end
	end
end

--UnitIsRaidOfficer

function cKick:CanShowAnchor()
	if ( _G[db].onlyRaid and self:IsInRaid() and (not self.Anchor.IsAnchorShown) ) then
		return true
	elseif ( not self.Anchor.IsAnchorShown ) then
		return true
	end
end


--------------------------------------
-- Raid roster handling
--------------------------------------
function cKick:HideButtons()
	self.Anchor.Reset:Hide()
	self.Anchor.Announce:Hide()
	self.Anchor.AddKicker:Hide()
	self.Anchor.RemKicker:Hide()
end

function cKick:ShowButtons()
	self.Anchor.Reset:Show()
	self.Anchor.Announce:Show()
	self.Anchor.AddKicker:Show()
	self.Anchor.RemKicker:Show()
end

function cKick:IsInRaid()
	if ( debug == true ) then
		self.debug("Debug modus enabled, fake raid")
		return true
	end

	if ( GetNumRaidMembers() > 0 ) then
		return true
	end
end

function cKick:RAID_ROSTER_UPDATE()
	--print("Raidlist update")
	if ( self:IsInRaid() ) then
		if ( not self.Anchor.Show ) then
			self.Anchor:Hide()
		else
			self.Anchor:Show()
		end

		for i = 1, 40 do
			unitName = select(1, UnitName("raid"..i))
			if ( unitName ) then
				_raidList[unitName] = "raid"..i
				--debug(unitName, tostring(_raidList[unitName]))
			end
		end
		
		if ( self:CanSendSync() ) then
			self:ShowButtons()
		else
			self:HideButtons()
		end
	else
		if ( _G[db].onlyRaid ) then
			self.Anchor:Hide()
		else
			self.Anchor:Show()
		end
		
		self:HideButtons()
	end
end

--------------------------------------
-- Addon communication
--------------------------------------
function cKick:AnnounceKicker()
	local syncString = _syncCmd.kickerSync
	
	for k, v in next, self.Bars do
		syncString = syncString .. " " .. k .. "_" .. cKick:GetKicker(k)
	end
	
	self.debug(syncString)
	SendAddonMessage(_syncCmd.prefix, syncString, "RAID")
end

function cKick:CHAT_MSG_ADDON(prefix, message, channel, sender)
	if ( (channel ~= "RAID") or (prefix ~= _syncCmd.prefix) ) then return end
	
	
	local argTbl = {}
	local command, args = strsplit(" ", message, 2)
	
	if ( args ) then
		argTbl = { strsplit(" ", args) }
	end
	
	if ( command == _syncCmd.kickerSync ) then
		self:RemoveAllKicker()
		
		for _, v in next, argTbl do
			local id, name = strsplit("_", v, 2)
			self:AddKicker(tonumber(id), name)
		end
		
		self:PlaceArrows(1, true)
		
	elseif ( command == _syncCmd.rotationReset ) then
		self:PlaceArrows(1, true)
		
	elseif ( command == _syncCmd.rotationSync ) then
		self:PlaceArrows(tonumber(argTbl[1]), true)
	end
end

--------------------------------------
-- Rotation handeling
--------------------------------------
function cKick:HideAllArrows()
	for i,_ in next, self.Bars do
		self.Bars[i]:HideArrows()
	end
end

function cKick:PlaceArrows(id, noAnnounce)
	if ( (not id) or (not self.Bars[id]) ) then return end
		
	self.debug("Place arrows on", id)
	self:HideAllArrows()
	self.Bars[id]:ShowArrows()
	--BigWigs:TriggerEvent("BigWigs_Message", self:GetKicker(id) .. " is next!", nil, true)
	
	if ( not noAnnounce ) then
		SendAddonMessage(_syncCmd.prefix, _syncCmd.rotationSync .. " " .. id, "RAID")
	end
end

function cKick:GetNextBarID(id)
	if ( not self.Bars[1] ) then return end
	
	if ( (not id) or (not self.Bars[id+1]) ) then
		return 1
	else
		return id + 1
	end
end

function cKick:ForwardRotation(id, depth)
	if ( not id ) then return end
	
	if ( not depth ) then
		depth = 1
	end
	
	if ( depth > #self.Bars ) then
		self.debug("Everyone is on CD, resetting rotation", depth)
		self:PlaceArrows(1, true)
		return
	end

	local nextID = self:GetNextBarID(id)
	self.debug("Next kicker:", nextID)
	
	if ( self.Bars[nextID].isRunning ) then -- Next kicker is on CD
		if ( self.Lockout.isRunning ) then -- Mob is still silenced
			if ( self.Bars[nextID].remaining <= self.Lockout.remaining ) then -- Kicker is ready before boss?
				self:PlaceArrows(nextID)
			else
				self:ForwardRotation(nextID, depth+1)
			end
		else
			self:ForwardRotation(nextID, depth+1)
		end
	else
		self:PlaceArrows(nextID)
	end
end

function cKick:COMBAT_LOG_EVENT_UNFILTERED(_, event, _, sourceName, _, _, destName, _, ...)
	if ( event == "SPELL_CAST_SUCCESS" ) then
		-- sourceSpellID, sourceSpellName, sourceSpellFlags
		local sourceSpellID, sourceSpellName, _ = ...
		local sourceUnitID = _raidList[sourceName]
		
		if ( not sourceUnitID ) then return end
		
		-- Do we have a class and was it an interrupt spell?
		local interrupterClass = select(2, UnitClass(sourceUnitID))
		if ( (not interrupterClass) or (not _kicks[interrupterClass]) or (not _kicks[interrupterClass][sourceSpellID]) ) then
			return
		end
		
		self.debug(sourceName, "used", sourceSpellID, sourceSpellName)
		
		-- Cooldown bar
		local id = self.Kicker[sourceName]
		local cooldownTime = _kicks[interrupterClass][sourceSpellID][1]
		local noCD = _kicks[interrupterClass][sourceSpellID][3]
		if ( id and cooldownTime and (not noCD)) then
			self.debug("Starting bar", id)
			self.Bars[id]:Start(cooldownTime, true)
			
			-- Arrow placing
			if ( self.Bars[id] and (self.Bars[id]:IsArrowShown()) ) then
				self:ForwardRotation(id)
			end
		end

	elseif ( event == "SPELL_INTERRUPT" ) then
		-- sourceSpellID, sourceSpellName, sourceSpellFlags, destSpellId, destSpellName, destSpellFlags
		local sourceSpellID, sourceSpellName, _, destSpellId, destSpellName, _ = ...
		local sourceUnitID = _raidList[sourceName]
		local focusName = select(1, UnitName("focus"))
		
		self.debug(destSpellId, destSpellName, "cast by", destName, "was interrupted by", sourceName, "with", sourceSpellID, sourceSpellName)
		
		if ( sourceUnitID and (destName == focusName) ) then
			-- Do we have a class and was it an interrupt spell?
			local interrupterClass = select(2, UnitClass(sourceUnitID))
			if ( (not interrupterClass) or (not _kicks[interrupterClass]) or (not _kicks[interrupterClass][sourceSpellID]) ) then
				self.debug("Interrupt spell could not be identified!", interrupterClass)
				return
			end
			
			-- Lockout bar
			local lockoutTime = _kicks[interrupterClass][sourceSpellID][2]
			if ( lockoutTime ) then
				_G[addon].debug("Starting lockout bar:", lockoutTime)
				self.Lockout:Start(lockoutTime, false)
			end
		end
	end
end

--------------------------------------
-- Bar handeling
--------------------------------------
function cKick:SortBars(id)
	if ( not id ) then
		id = 1
	end
	
	if ( not self.Bars[id] ) then
		local name = self:GetKicker(id + 1)
		
		if ( name ) then
			self:RemoveKicker(id + 1, true)
			self:AddKicker(id, name)
		end
	end
	
	if ( self.Bars[id+2] or (not self.Bars[id+2] and self.Bars[id+1]) ) then
		self:SortBars(id + 1)
	end
end

function cKick:MoveBars(id, arg)
	if ( not id ) then
		id = 1
	end
	
	local temp = self:GetKicker(id)
	
	if ( self.Bars[id] ) then
		self:RemoveKicker(id, true)
	end
	
	if ( arg ) then
		self:AddKicker(id, arg)
	end
	
	if ( temp ) then
		self:MoveBars(id + 1, temp)
	end
end

function cKick:GetKicker(arg)
	if ( type(arg) == "number" ) then
		for k, v in next, self.Kicker do
			if ( v == arg ) then
				return k
			end
		end
	elseif ( type(arg) == "string" ) then
		for k, v in next, self.Kicker do
			if ( k == arg ) then
				return v
			end
		end
	end
end

function cKick:RemoveKicker(arg, noSort)
	if ( type(arg) == "number" ) then
		local name = self:GetKicker(arg)
		self.Kicker[name] = nil
		self.Bars[arg]:Hide()
		self.Bars[arg]:ClearAllPoints()
		self.Bars[arg] = nil
	elseif ( type(arg) == "string" ) then
		local id = self:GetKicker(arg)
		self.Kicker[arg] = nil
		self.Bars[id]:Hide()
		self.Bars[id]:ClearAllPoints()
		self.Bars[id] = nil
	end
	
	if ( not noSort ) then
		self:SortBars()
	end
end

function cKick:RemoveAllKicker()
	for k,_ in next, self.Bars do
		self:RemoveKicker(k, true)
	end
end

-- Debug funcs
function cKick:AddRaid()
	for i = 1, GetNumRaidMembers() do
		self:AddKicker(i, UnitName("raid" .. i))
	end
end

function cKick:AddTestKicker()
	self:AddKicker(1, "Chimist", "player")
	self:AddKicker(2, "Chim", "player")
	self:AddKicker(3, "Chimaine", "player")
	self:AddKicker(4, "Tsuyoki", "player")
	self:AddKicker(5, "Benihime", "player")
end


function cKick:AddKicker(id, name, unit)
	if ( not name ) then
		name = UnitName("target")
	end
	
	if ( (#name <= 2) or (not id) ) then
		self.print("Cannot add kicker: No valid name or id given!")
		return
	end

	local unitID = _raidList[name]
	
	if ( unit ) then
		unitID = unit
	end
	
	if ( not unitID ) then
		self.print(name, "is not a member of your raid! ", unitID, unit)
		return
	end
	
	if ( self.Kicker[name] ) then
		self:RemoveKicker(name)
	end
	
	if ( self.Bars[id] ) then
		self:MoveBars(id)
	end
	
	local color
	
	if ( _G[db].useClass ) then
		local class = select(2, UnitClass(unitID))
		color = { RAID_CLASS_COLORS[class].r, RAID_CLASS_COLORS[class].g, RAID_CLASS_COLORS[class].b }
	else
		color = _G[db].barColor
	end

	self.Kicker[name] = id
	
	self.Bars[id] = self:CreateBar(_G[db].barWidth, _G[db].barHeight)
	self.Bars[id]:SetParent(self.Anchor)
	self.Bars[id]:SetLabel(id .. ": " .. name)
	self.Bars[id]:SetTextStyle(LSM:Fetch("font", _G[db].fontFace), _G[db].fontSize)
	self.Bars[id]:SetColor(unpack(color))
	
	local space = (( _G[db].barHeight + _G[db].barSpacing) * (id-1)) + 3
	self.Bars[id]:SetPoint("TOP", self.Lockout, "BOTTOM", 0, -space)
end

function cKick:Update()
	self.Anchor:SetWidth(_G[db].barWidth)
	
	self.Anchor:SetScale(_G[db].scale)
	
	self.Anchor:SetMovable(not _G[db].lockFrame)
	--self.Anchor:EnableMouse(not _G[db].lockFrame)
	
	self.Anchor.Label:SetFont(LSM:Fetch("font", _G[db].fontFac), 11, select(3, self.Anchor.Label:GetFont()))

	self.Lockout:Hide()
	self.Lockout = wipe(self.Lockout)
	self:CreateLockoutBar()

	for k, _ in next, self.Bars do
		self:AddKicker(k, self:GetKicker(k))
	end
end

function cKick:SetupAnchor()
	-- Anchor
	local anchor = CreateFrame("Frame", "cKick_Anchor", UIParent)
	
	anchor:SetWidth(_G[db].barWidth)
	anchor:SetHeight(15)
	
	anchor:SetScale(1)
	anchor:SetAlpha(1)
		
	anchor:SetMovable(1)
	anchor:EnableMouse(1)
	anchor:SetUserPlaced(0)
	anchor:SetClampedToScreen(1)
	
	anchor:SetPoint(_G[db].point[1], UIParent, _G[db].point[3], _G[db].point[4], _G[db].point[5])

	anchor:SetScript("OnMouseDown", function()
		anchor:StartMoving()
	end)
	anchor:SetScript("OnMouseUp", function()
		anchor:StopMovingOrSizing()
		local point, parent, relativeTo, xPos, yPos = anchor:GetPoint()
		_G[db].point = { point, parent, relativeTo, xPos, yPos }
	end)
	
	anchor:SetScript("OnShow", function() anchor.IsAnchorShown = true end)
	anchor:SetScript("OnHide", function() anchor.IsAnchorShown = false end)

	local label = anchor:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmallOutline")
	label:SetPoint("CENTER", anchor, 0, 0)
	label:SetFont(LSM:Fetch("font", _G[db].fontFace), 11, select(3, label:GetFont()))
	label:SetText(addon)
	anchor.Label = label
	
	-- Buttons
	local reset = CreateFrame("Button", nil, anchor)
	reset:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
	reset:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")	
	reset:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	reset:SetWidth(14)
	reset:SetHeight(14)
	reset:SetPoint("RIGHT", anchor, "RIGHT", -10, 0)
	reset:SetScript("OnClick", function()
		if ( self:CanSendSync() ) then
			self.debug("Sending reset")
			SendAddonMessage(_syncCmd.prefix, _syncCmd.rotationReset, "RAID")
		end
	end)
	reset:SetFrameLevel(reset:GetFrameLevel() + 1)
	anchor.Reset = reset
	
	local announce = CreateFrame("Button", nil, anchor)
	announce:SetNormalTexture("Interface\\Buttons\\UI-GuildButton-MOTD-Up")
	announce:SetPushedTexture("Interface\\Buttons\\UI-GuildButton-MOTD-Down")	
	announce:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	announce:SetWidth(12)
	announce:SetHeight(12)
	announce:SetPoint("RIGHT", anchor.Reset, "LEFT", -1, 0)
	announce:SetScript("OnClick", function()
		if ( self:CanSendSync() ) then
			self:AnnounceKicker()
		end
	end)
	announce:SetFrameLevel(announce:GetFrameLevel() + 1)
	anchor.Announce = announce

	local addKicker = CreateFrame("Button", nil, anchor)
	addKicker:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up")
	addKicker:SetPushedTexture("Interface\\Buttons\\UI-PlusButton-Down")	
	addKicker:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	addKicker:SetWidth(11)
	addKicker:SetHeight(11)
	addKicker:SetPoint("LEFT", anchor, "LEFT", 10, 0)
	addKicker:SetScript("OnClick", function()
		if ( self:CanSendSync() ) then
			self.Popup:ShowPopup("add")
		end
	end)
	addKicker:SetFrameLevel(addKicker:GetFrameLevel() + 1)
	anchor.AddKicker = addKicker
	
	local remKicker = CreateFrame("Button", nil, anchor)
	remKicker:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up")
	remKicker:SetPushedTexture("Interface\\Buttons\\UI-MinusButton-Down")	
	remKicker:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	remKicker:SetWidth(11)
	remKicker:SetHeight(11)
	remKicker:SetPoint("LEFT", anchor.AddKicker, "RIGHT", 1, 0)
	remKicker:SetScript("OnClick", function()
		if ( self:CanSendSync() ) then
			self.Popup:ShowPopup("rem")
		end
	end)
	remKicker:SetFrameLevel(remKicker:GetFrameLevel() + 1)
	anchor.RemKicker = remKicker
	
	self.Anchor = anchor
end

function cKick:CreateLockoutBar()
	-- Lockout timerbar
	self.Lockout = self:CreateBar(_G[db].barWidth, _G[db].barHeight)
	self.Lockout:SetParent(self.Anchor)
	self.Lockout:SetPoint("TOP", self.Anchor, "BOTTOM")
	self.Lockout:SetLabel("Lockout")
	self.Lockout:SetTextStyle(LSM:Fetch("font", _G[db].fontFace), _G[db].fontSize)
	self.Lockout:SetColor(unpack(_G[db].barColor))
	self.Lockout.Bar:SetMinMaxValues(0, 1)
	self.Lockout.Bar:SetValue(0)
end

function cKick:PLAYER_LOGIN()
	self:SetupAnchor()
	self:SetupPopup()
	self:CreateLockoutBar()
	self:Update()
	
	self.Anchor:Hide()
	
	self:RAID_ROSTER_UPDATE()
	self:RegisterEvent("CHAT_MSG_ADDON")
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	
	-- Cleanup
	self:UnregisterEvent("PLAYER_LOGIN")
	self.SetupAnchor = nil
	self.SetupPopup = nil
	self.ADDON_LOADED = nil
	self.PLAYER_LOGIN = nil
end

function cKick:ADDON_LOADED(arg)
	if ( tostring(arg1) ~= addon ) then return end
	
	self:UnregisterEvent("ADDON_LOADED")
	
	self.DB = cDatabase:NewInstance(addon, db)
	self.DB:Create()
	self.DB:AddSlashCommand(self.Name, "/ckick")
	
	self:RegisterEvent("PLAYER_LOGIN")
end

-- Config helper
function cKick:Defaults()
	return {
		lockFrame = false,
		onlyRaid = true,
		
		scale = 1,
		point = {"CENTER", UIParent, "CENTER", 0, 0},
		
		barWidth = 150,
		barHeight = 15,
		barSpacing = 1,
		barTexture = "darkborder",
		
		barColor = {.31, .41, .53},
		classColor = false,
		
		fontSize = 10,
		fontFace = "Friz Quadrata TT",
		
		DBVersion = 0,
	}
end

function cKick:FrameLock()
	self.Anchor:SetMovable(not _G[db].lockFrame)
	self.Anchor:EnableMouse(not _G[db].lockFrame)
end

-- Slash command
function cKick:SlashCMD(cmd)
	if ( cmd == "debug" ) then
		debug = not debug
		return
	elseif ( cmd == "config" or cmd == "options" ) then
		local loaded, reason = LoadAddOn(addon .. "_Config")
		if ( not loaded ) then
			self.print("Config could not be loaded:", reason)
		else
			InterfaceOptionsFrame_OpenToCategory(_G[addon].Name)
		end
		return
	elseif ( cmd == "lock" ) then
		_G[db].lockFrame = not _G[db].lockFrame
		_G[addon]:FrameLock()
		return
	elseif ( cmd == "toggle" ) then
		if ( _G[addon].Anchor:IsVisible() ) then
			_G[addon].Anchor:Hide()
			_G[addon].Popup:Hide()
		else
			_G[addon].Anchor:Show()
		end
		return
	else
		self.print("Valid commands are: \"config\", \"toggle\", \"lock\" and \"debug\"")
	end
end

_G[addon]:RegisterEvent("ADDON_LOADED")
_G[addon]:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)
