-- Database for cAddons

-- NOTE: You NEED to define
-- Addon:Defaults()
-- Addon:SlashCMD(cmd)

local _revision = 8

if ( cDatabase and cDatabase.revision >= _revision ) then
	return
end

if ( not tobool ) then
	-- Since standart wow lua does not provide a tobool func we have to make one
	-- Returns matching true or false for strings and numbers, or returns the input if no match was found or type was not string/number
	-- TODO: Convert single entry tables/arrays
	function tobool(arg)
		local sourceType = type(arg)
		
		if ( sourceType == "string" ) then
			if ( arg == "true" ) then
				return true
			elseif ( arg == "false" ) then
				return false
			end
		elseif ( sourceType == "number" ) then
			if ( arg >= 1 ) then
				return true
			elseif ( arg == 0 ) then
				return false
			end
		end
		
		return arg
	end
	
end

cDatabase = {}
cDatabase.revision = _revision

local prototype = {}
prototype.__index = prototype

-- Create a new DB or check for existing ones and clean them if versions differ
function prototype:Create()
	if ( _G[self.db] ) then
		if ( _G[self.db].DBVersion ~= _G[self.addon].DBVersion ) then
			_G[self.addon].debug("Cleaning DB")
			_G[self.db] = self:Clean(_G[self.db], _G[self.addon]:Defaults())
			_G[self.db].DBVersion = _G[self.addon].DBVersion
		end
	else
		_G[self.addon].debug("Creating DB")
		self:Reset()
	end
end

-- Reset a db to defaults
function prototype:Reset()
	_G[self.db] = {}
	_G[self.db] = _G[self.addon]:Defaults()
	_G[self.db].DBVersion = _G[self.addon].DBVersion
end

-- Recursive table clean func
function prototype:Clean(configTable, defaultsTable, level)
	local level = level or 0
	
	local ln = ""
	for i = 0, level do
		ln = ln .. "- "
	end

	for k, _ in pairs(configTable) do
		if ( defaultsTable[k] == nil ) then -- unessecary entry
			configTable[k] = nil
			_G[self.addon].debug(ln .. "Deleted entry \"" .. k .. "\"")
		else
			local defaults_type = type(defaultsTable[k])
			local target_type = type(configTable[k])
			
			if ( defaults_type == "table" ) then -- table, recursive call
				_G[self.addon].debug(ln .. "Going to table \"" .. k .. "\"")
				configTable[k] = self:Clean(configTable[k], defaultsTable[k], level+1)
			elseif ( defaults_type ~= target_type ) then -- entry types dont match
				if ( defaults_type == "number" or defaults_type == "string" or defaults_type == "bool" ) then -- convert string to number and vice versa. other types will fail to convert. TODO: tobool()
					_G[self.addon].debug(ln .. "Converting entry \"" .. k .. "\"")
					configTable[k] = _G("to" .. defaults_type)(configTable[k])
				else
					_G[self.addon].debug(ln .. "Overwriting entry \"" .. k .. "\"")
					configTable[k] = defaultsTable[k]
				end
			end
		end
	end
		
	for k, _ in pairs(defaultsTable) do
		if ( configTable[k] == nil ) then -- missing entry
			configTable[k] = defaultsTable[k]
			_G[self.addon].debug(ln .. "Added entry \"" .. k .. "\"")
		end
	end

	return configTable
end

-- Slash commands
function prototype:AddSlashCommand(name, ...)
	local num = 0
	for i = 1, select('#', ...) do
		local cmd = select(i, ...)
		num = num + 1
		_G["SLASH_" .. name .. num] = cmd
	end

	_G["hash_SlashCmdList"][name] = nil
	_G["SlashCmdList"][name] = function(cmd) _G[self.addon]:SlashCMD(cmd) end
end
	
function cDatabase:NewInstance(addon, db)
	if ( (not _G[addon].Defaults) or (not _G[addon].SlashCMD) ) then
		return error("cDatabase r" .. revision .. " couldn't find defaults or slash command function")
	end
	
	_G[addon].debug("Opening DB file")
	
	local lib = {}
	lib.addon = addon
	lib.db = db
	
	setmetatable(lib, prototype)
	
	return lib
end
