--[[
	cKick Config
	Copyright(c) 2009, Tobias 'Chimaine' Rummelt, postmaster(at)tobis-little-page.de
	Chimaine on EU-Destromath
	All rights reserved.
]]

local addon, db = "cKick", "cKickDB"

local _G = getfenv(0)

if ( not _G[addon] or not _G[db] ) then
	return
end

local unpack = _G.unpack

local function SetFunc(info, val)
	_G[db][info[#info]] = val
	_G[addon]:Update()
end

local function GetFunc(info)
	return _G[db][info[#info]]
end

local function SetColorFunc(info, r, g, b, a)
	_G[db][info[#info]] = {r, g, b, a}
	_G[addon]:Update()
end

local function GetColorFunc(info)
	return unpack(_G[db][info[#info]])
end

local LSM = LibStub("LibSharedMedia-3.0")
local AC = LibStub("AceConfig-3.0")

AC:RegisterOptionsTable(addon.."_Config", {
	name = _G[addon].Name .. " Options",
	type = "group",
	childGroups = "tab",
	args = {
		general_options = {
			name = "General",
			order = 1,
			type = "group",
			args = {
				lockFrame = {
					order = 11,
					type = "toggle",
					name = "Lock frame",
					desc = "Locks the cKick frame and bars in place.",
					get = GetFunc,
					set = function(info, val) SetFunc(info, val); _G[addon]:FrameLock() end,
				},
				onlyRaid = {
					order = 12,
					type = "toggle",
					name = "Only in Raid",
					desc = "Automaticly hides the bars when you are not in a raid.",
					get = GetFunc,
					set = function(info, val) SetFunc(info, val); if ( not val and _G[addon]:CanShowAnchor() ) then _G[addon].Anchor:Show() else _G[addon].Anchor:Hide() end end,
				},
				defaults = {
					order = 13,
					type = "execute",
					name = "Defaults",
					desc = "Resets the options to their default values.",
					func = function() _G[addon].DB:Reset(); _G[addon]:Update(); ReloadUI() end,
					confirm = true,
					confirmText = "Restore defaults? This will reload your UI.",
					width = "full",
				},
			},
		},
		appearance_options = {
			name = "Appearance",
			order = 2,
			type = "group",
			childGroups = "tab",
			args = {
				scale = {
					order = 21,
					type = "range",
					name = "Scale",
					desc = "The scale of the frame and the bars.",
					min = 0.1,
					max = 2,
					step = 0.1,
					get = GetFunc,
					set = SetFunc,
				},
				barWidth = {
					order = 22,
					type = "range",
					name = "Bar width",
					desc = "The width of the bars.",
					min = 50,
					max = 400,
					step = 1,
					get = GetFunc,
					set = SetFunc,
				},
				barHeight = {
					order = 23,
					type = "range",
					name = "Bar height",
					desc = "The height of the bars.",
					min = 5,
					max = 50,
					step = 1,
					get = GetFunc,
					set = SetFunc,
				},
				barSpacing = {
					order = 24,
					type = "range",
					name = "Bar spacing",
					desc = "The height of the space between the bars.",
					min = 0,
					max = 10,
					step = 1,
					get = GetFunc,
					set = SetFunc,
				},
				barTexture = {
					order = 25,
					type = "select",
					dialogControl = "LSM30_Statusbar", --Select your widget here
					name = "Bar texture",
					desc = "Texture of the bars.",
					values = LSM:HashTable("statusbar"), -- pull in your font list from LSM
					get = GetFunc,
					set = SetFunc,
				},
			},
		},
		color_options = {
			name = "Color",
			order = 3,
			type = "group",
			childGroups = "tab",
			args = {
				barColor = {
					order = 31,
					type = "color",
					name = "Bar color",
					desc = "Standart color of the bars.",
					hasAlpha = false,
					get = GetColorFunc,
					set = SetColorFunc,
				},
				classColor = {
					order = 32,
					type = "toggle",
					name = "Use class colors",
					desc = "Use class colors for individual bars or the standart color.",
					get = GetFunc,
					set = SetFunc,
				},
			},
		},
		font_options = {
			name = "Font",
			order = 4,
			type = "group",
			childGroups = "tab",
			args = {
				fontSize = {
					order = 41,
					type = "range",
					name = "Font size",
					desc = "The size of the font.",
					min = 1,
					max = 30,
					step = 1,
					get = GetFunc,
					set = SetFunc,
				},
				fontFace = {
					order = 42,
					type = 'select',
					dialogControl = 'LSM30_Font', --Select your widget here
					name = "Font face",
					desc = "Style of the font.",
					values = LSM:HashTable("font"), -- pull in your font list from LSM
					get = GetFunc,
					set = SetFunc,
				},
			},
		},
	},
})

LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addon.."_Config", _G[addon].Name)
